
#define i8 int8_t
#define u8 uint8_t
#define i16 int16_t
#define u16 uint16_t
#define i32 int32_t
#define u32 uint32_t

#define lo8(n) (n&0xff)
#define hi8(n) ((n>>8)&0xff)

#define CONCAT__2(a, b)		a ## b
#define CONCAT(a, b)		CONCAT__2(a, b)
#define CONCAT__3(a, b, c)	a ## b ## c
#define CONCAT3(a, b, c)	CONCAT__3(a, b, c)

#define CON_OUT(name)		CONCAT(PORT, name)
#define CON_IN(name)		CONCAT(PIN, name)
#define CON_DDR(name)		CONCAT(DDR, name)

#define min(a, b)		(a<b ? a : b)
#define max(a, b)		(a>b ? a : b)


/*****************************************************/

#define BAUDRATE	115200


#define SCOPE_SIZE	100
#define SCOPE_TRIGGER	128	// Trigger-Level 1-255


#define LED0_PORT_NAME	B
#define LED0_BIT_NO	5


// Generator/PWM-Output
#define PWM_TIMER_NO	1	// Must be 16 bit timer
#define PWM_COMP_CH	A	// Comparator channel
#define PWM_PORT_NAME	B	// Output-Port
#define PWM_BIT_NO	1	// Output-Bit-No


// Scope/ADC-Input
#define ADC_REF_BITS	(1<<REFS1)|(1<<REFS0)	// Internal 1.1V
//#define ADC_REF_BITS		(1<<REFS0)	// Avcc (1.8-6V, usually 3.3 or 5V)

#define ADC_PRESC_BITS		(7<<ADPS0)	// Prescaler 128 - for above 12.8 MHz
//#define ADC_PRESC_BITS		(6<<ADPS0)	// Prescaler  64 - for 12.8 MHz and below
//#define ADC_PRESC_BITS		(5<<ADPS0)	// Prescaler  32 - for  6.4 MHz and below
//#define ADC_PRESC_BITS		(4<<ADPS0)	// Prescaler  16 - for  3.2 MHz and below
//#define ADC_PRESC_BITS		(3<<ADPS0)	// Prescaler   8 - for  1.6 MHz and below
//#define ADC_PRESC_BITS		(2<<ADPS0)	// Prescaler   4 - for  800 KHz and below
//#define ADC_PRESC_BITS		(1<<ADPS0)	// Prescaler   2 - for  400 MHz and below

#define ADC_IN_BITS	0	// Input Select (0..7), 8=Temp. Sensor, 14=1.1 V, 15=0 V

#define ADC_TIMER_NO	0
#define ADC_COMP_CH	A	// Comparator channel
#define ADC_PORT_NAME	C	// Output-Port
#define ADC_BIT_NO	ADC_IN_BITS	// Output-Bit-No


#define VUC_PUT_ADR_MAX	0	// 1 input
#define VUC_GET_ADR_MAX	0	// 1 output


// "put"-commands:
u8 put_timer (void);

// "get"-commands:
u8 get_scope (void);


/*****************************************************/

#define usleep(us)	_delay_loop_2( (F_CPU/4) / (us*1000000UL) );	// 1 tick = F_CPU/4 = 250nS@16MHz

#define LED0_PORT_ADR	CON_OUT(LED0_PORT_NAME)
#define LED0_DDR_ADR	CON_DDR(LED0_PORT_NAME)

#define PWM_PORT_ADR	CON_OUT(PWM_PORT_NAME)
#define PWM_DDR_ADR	CON_DDR(PWM_PORT_NAME)

#define ADC_PORT_ADR	CON_OUT(ADC_PORT_NAME)
#define ADC_DDR_ADR	CON_DDR(ADC_PORT_NAME)


#if (PWM_COMP_CH==A)
#  define PWM_TOP_CH	B
#else
#  define PWM_TOP_CH	A
#endif
#define PWM_OUTPUT	CONCAT(PWM_TIMER_NO, PWM_COMP_CH)
#define PWM_ICR		CONCAT(ICR, PWM_TIMER_NO)
#define PWM_OCR		CONCAT(OCR, PWM_OUTPUT)
#define PWM_TCNT	CONCAT(TCNT, PWM_TIMER_NO)
#define PWM_TCCR_A	CONCAT3(TCCR, PWM_TIMER_NO, A)
#define PWM_TCCR_B	CONCAT3(TCCR, PWM_TIMER_NO, B)
#define PWM_OSIG	CONCAT3(CONCAT(TIMER, PWM_TIMER_NO), CONCAT(_COMP, PWM_COMP_CH), _vect)
#define PWM_ICNC	CONCAT(ICNC, PWM_TIMER_NO)
#define PWM_CS_0	CONCAT3(CS, PWM_TIMER_NO, 0)
#define PWM_CS_1	CONCAT3(CS, PWM_TIMER_NO, 1)
#define PWM_CS_2	CONCAT3(CS, PWM_TIMER_NO, 2)
#define PWM_WGM_0	CONCAT3(WGM, PWM_TIMER_NO, 0)
#define PWM_WGM_1	CONCAT3(WGM, PWM_TIMER_NO, 1)
#define PWM_WGM_2	CONCAT3(WGM, PWM_TIMER_NO, 2)
#define PWM_WGM_3	CONCAT3(WGM, PWM_TIMER_NO, 3)
#define PWM_ICF		CONCAT(ICF, PWM_TIMER_NO)
#define PWM_OCF		CONCAT3(OCF, PWM_TIMER_NO, PWM_COMP_CH)
#define PWM_COM_0	CONCAT3(COM, PWM_OUTPUT, 0)
#define PWM_COM_1	CONCAT3(COM, PWM_OUTPUT, 1)
#define PWM_OCIE	CONCAT(OCIE, PWM_OUTPUT)


#define ADC_TCCR_A	CONCAT3(TCCR, ADC_TIMER_NO, A)
#define ADC_WGM_0	CONCAT3(WGM, ADC_TIMER_NO, 0)
#define ADC_WGM_1	CONCAT3(WGM, ADC_TIMER_NO, 1)
#define ADC_WGM_2	CONCAT3(WGM, ADC_TIMER_NO, 2)
#define ADC_WGM_3	CONCAT3(WGM, ADC_TIMER_NO, 3)
#define ADC_TCCR_B	CONCAT3(TCCR, ADC_TIMER_NO, B)
#define ADC_CS_0	CONCAT3(CS, ADC_TIMER_NO, 0)
#define ADC_CS_1	CONCAT3(CS, ADC_TIMER_NO, 1)
#define ADC_CS_2	CONCAT3(CS, ADC_TIMER_NO, 2)
#define ADC_ICR		CONCAT(ICR, ADC_TIMER_NO)
#define ADC_OCR		CONCAT3(OCR, ADC_TIMER_NO, ADC_COMP_CH)
#define ADC_TIMSK	CONCAT(TIMSK, ADC_TIMER_NO)
#define ADC_OCIE	CONCAT3(OCIE, ADC_TIMER_NO, ADC_COMP_CH)
#define ADC_TIFR	CONCAT(TIFR, ADC_TIMER_NO)
#define ADC_OCF		CONCAT3(OCF, ADC_TIMER_NO, ADC_COMP_CH)


/*****************************************************/

#define VUC_PROTO_VER	"0.1"	// Protocol version

// Commands from control-panel
#define VUC_CMD_ASCII	0	// Ascii-data channel for extd. cmds
#define VUC_CMD_PUT	1	// Set value(s)
#define VUC_CMD_GET	2	// Get Value(s)
#define VUC_CMD__MAX	2

// replies from uC
#define VUC_REP_ASCII	0	// dito
#define VUC_REP_STATUS	1	// done
#define VUC_REP_PUT	2	// wtf
#define VUC_REP__MAX	2

#if (VUC_PUT_ADR_MAX<=0xff && VUC_GET_ADR_MAX<=0xff)
typedef u8 vuc_adr_t;
#else
typedef u16 vuc_adr_t;
#endif

typedef u8 (* vuc_put_f)(void);
typedef u8 (* vuc_get_f)(void);

typedef struct {
  u8 no_of_bytes;
  vuc_put_f func;
} vuc_put_meta_t;

typedef struct {
  vuc_get_f func;
} vuc_get_meta_t;

